/*
 * Copyright (c) 2008 - 2013  Extensoft, Inc.
 * All Rights Reserved.
 *
 * For redistribution and licensing information please visit: http://www.artisteer.com/?p=code_license_agreement
 * For support and help, please visit our developer's section: http://www.artisteer.com/?p=dev
 *
 * Do NOT edit this file as it is auto-generated by Artisteer.
*/

/*jshint forin:true, noarg:true, noempty:true, eqeqeq:true, bitwise:true, strict:true, undef:true, curly:true, browser:true, jquery:true, devel:true */
(function() {
    'use strict';
    var _fileManagerStorages = [];
    window.FileManager = function(fileStorage) {
        var _fileStorage;
        // Private functions

        function clone(obj) {
            var newObj, property;
            if (!obj || 'object' !== typeof obj) {
                return obj;
            }
            newObj = { };
            for (property in obj) {
                if (obj.hasOwnProperty(property)) {
                    newObj[property] = clone(obj[property]);
                }
            }
            return newObj;
        }

        function getPathParts(path) {
            var pathPartsTmp = path.split( /[\\\/]/ ), pathParts = [], i;
            for (i = 0; i < pathPartsTmp.length; i++) {
                if (pathPartsTmp[i] !== '') {
                    pathParts.push(pathPartsTmp[i]);
                }
            }
            return pathParts;
        }

        function isParentPath(parentPath, childrenPath) {
            var parentPathParts, childrenPathParts, i;
            parentPathParts = getPathParts(parentPath);
            childrenPathParts = getPathParts(childrenPath);
            if (parentPathParts.length > childrenPathParts.length) {
                return false;
            }
            for (i = 0; i < parentPathParts.length; i++) {
                if (parentPathParts[i] !== childrenPathParts[i]) {
                    return false;
                }
            }
            return true;
        }

        function getObjectByPath(storage, path) {
            var pathParts, obj, i;
            if (arguments.length === 1) {
                path = storage;
                storage = _fileStorage;
            }
            pathParts = getPathParts(path);
            obj = storage;
            for (i = 0; i < pathParts.length; i++) {
                if (obj.items && obj.items[pathParts[i]]) {
                    obj = obj.items[pathParts[i]];
                } else {
                    return null;
                }
            }
            return obj;
        }

        // End of private functions
        if (fileStorage) {
            _fileStorage = clone(fileStorage);
        } else {
            _fileStorage = { type: 'dir', items: { } };
        }
        _fileManagerStorages.push([this, _fileStorage]);
        /**
         * Checks if the file exists in the specified directory
         * 
         * @param {String} path A path to a folder or directory
         * @returns {Boolean} Returns "true" if a file or directory defined by path parameter exists, otherwise returns "false"
         */
        this.exists = function(path) {
            if (getObjectByPath(path)) {
                return true;
            } else {
                return false;
            }
        };
        /**
         * Copies a file or directory
         * 
         * @param {String} src A path to a source file or directory
         * @param {String} dest A path to a destination file or directory
         */
        this.copy = function(src, dest) {
            var destParts, destParentDir, srcObj, file;
            if (isParentPath(src, dest)) {
                console.warn('Unable to copy the parent directory "' + src + '"" to the child directory "' + dest + '".');
                return;
            }
            if (this.exists(src)) {
                if (this.isFile(src)) {
                    destParts = getPathParts(dest);
                    if (!destParts.length) {
                        console.warn('Unable to replace the the root directory with "' + src + '" file.');
                        return;
                    }
                    destParentDir = destParts.slice(0, -1).join('/');
                    this.createDirectory(destParentDir);
                    getObjectByPath(destParentDir).items[destParts[destParts.length - 1]] = clone(getObjectByPath(src));
                } else if (this.isDirectory(src)) {
                    srcObj = getObjectByPath(src);
                    if (srcObj.type === 'dir') {
                        this.createDirectory(dest);
                    } else {
                        this.createArchive(dest, srcObj.type);
                    }
                    for (file in srcObj.items) {
                        if (srcObj.items.hasOwnProperty(file)) {
                            this.copy(src + '/' + file, dest + '/' + file);
                        }
                    }
                }
            } else {
                console.warn('The source file "' + src + '" does not exist.');
                return;
            }
        };
        /**
         * Moves a file or directory 
         * 
         * @param {String} src A path to a source file or directory
         * @param {String} dest A path to a destination file or directory
         */
        this.move = function(src, dest) {
            var destParentDir, destParts, srcObj, file;
            if (isParentPath(src, dest)) {
                console.warn('Unable to move the parent directory "' + src + '" to the child directory "' + dest + '".');
                return;
            }
            if (this.exists(src)) {
                if (this.isFile(src)) {
                    destParts = getPathParts(dest);
                    if (!destParts.length) {
                        console.warn('Unable to replace the root directory with "' + src + '" file.');
                        return;
                    }
                    destParentDir = destParts.slice(0, -1).join('/');
                    this.createDirectory(destParentDir);
                    getObjectByPath(destParentDir).items[destParts[destParts.length - 1]] = clone(getObjectByPath(src));
                    this.remove(src);
                } else if (this.isDirectory(src)) {
                    srcObj = getObjectByPath(src);
                    if (srcObj.type === 'dir') {
                        this.createDirectory(dest);
                    } else {
                        this.createArchive(dest, srcObj.type);
                    }
                    for (file in srcObj.items) {
                        if (srcObj.items.hasOwnProperty(file)) {
                            this.move(src + '/' + file, dest + '/' + file);
                        }
                    }
                    this.remove(src);
                }
            } else {
                console.warn('The souce file "' + src + '" does not exist.');
                return;
            }
        };
        /**
         * Creates a directory in the specified path
         * 
         * @param {String} path A Path to a created folder
         */
        this.createDirectory = function(path) {
            var pathParts, dir, i, j, fileName;
            pathParts = getPathParts(path);
            dir = _fileStorage;
            for (i = 0; i < pathParts.length; i++) {
                if (dir.items && dir.items[pathParts[i]]) {
                    if (!dir.items[pathParts[i]].items) {
                        fileName = '';
                        for (j = 0; j <= i; j++) {
                            fileName += '/' + pathParts[j];
                        }
                        console.warn('An a attempt was made to create a sub folder in the file "' + fileName + '". This forced the file to be deleted.');
                        delete dir.items[pathParts[i]];
                        dir.items[pathParts[i]] = { type: 'dir', items: { } };
                    }
                } else {
                    dir.items[pathParts[i]] = { type: 'dir', items: { } };
                }
                dir = dir.items[pathParts[i]];
            }
        };
        /**
         * Creates an archive direcory, which will be archived by Artisteer
         * 
         * @param {String} path A path to a created directory
         * @param {String} type Supported archive formats: zip, gz, bz, cab. The default format - zip
         */
        this.createArchive = function(path, type) {
            var archive;
            type = type || 'zip';
            type = (['zip', 'gz', 'bz', 'cab'].indexOf(type) !== -1) ? type : 'zip';
            this.createDirectory(path);
            archive = getObjectByPath(path);
            archive.type = type;
        };
        /**
         * Deletes a file or directory
         * 
         * @param {String} path A path to the deleted file or directory
         */
        this.remove = function(path) {
            var pathParts, parentDir;
            pathParts = getPathParts(path);
            if (!pathParts.length) {
                console.warn('The root directory cannot be deleted.');
                return;
            }
            if (!getObjectByPath(path)) {
                console.warn('The deleted "' + path + '" file does not exist.');
                return;
            }
            parentDir = getObjectByPath(pathParts.slice(0, -1).join('/'));
            if (parentDir && parentDir.items) {
                delete parentDir.items[pathParts[pathParts.length - 1]];
            }
        };
        /**
         * Reads the file content into a string
         * 
         * @param {String} path  A path to the read file
         * @returns {String} File content
         */
        this.read = function(path) {
            var file = getObjectByPath(path);
            if (file && file.type === 'text') {
                return file.content;
            }
            if (!this.exists(path)) {
                console.warn('The file "' + path + '" does not exist.');
                return '';
            }
            if (this.isBinary(path)) {
                console.warn('Unable to read the binary file "' + path + '".');
                return '';
            }
            if (this.isDirectory(path)) {
                console.warn('Unable to read the "' + path + '" directory.');
                return '';
            }
        };
        /**
         * Writes text to a file
         * 
         * @param {String} path A path to the writen file
         * @param {String} content The content being writen
         */
        this.write = function(path, content) {
            var pathParts, parentDir, parentDirObject;
            pathParts = getPathParts(path);
            if (!pathParts.length) {
                console.warn('Unable to write in the root directory. This is not a file.');
                return;
            }
            if (this.isBinary(path)) {
                console.warn('Unable to write in the "' + path + '" binary file.');
                return;
            }
            parentDir = pathParts.slice(0, -1).join('/');
            this.createDirectory(parentDir);
            parentDirObject = getObjectByPath(parentDir);
            parentDirObject.items[pathParts[pathParts.length - 1]] = { type: 'text', content: content };
        };
        /**
         * Appends a content to the end of the file
         * 
         * @param {String} path A path to the appended file
         * @param {String} content The appended content
         */
        this.append = function(path, content) {
            this.write(path, this.read(path) + content);
        };

        /**
         * Returns the list of files and directories in the specified path (not recursively)
         * 
         * @param {String} path A path to the read directory 
         * @returns {Array} An array of the first level child files and directories
         */
        this.enumerate = function(path) {
            var dirObject, dirList = [], fileList = [], name;
            dirObject = getObjectByPath(path);
            if (!dirObject) {
                console.warn('Directory "' + path + '" does not exist.');
                return [];
            }
            if (dirObject && !dirObject.items) {
                console.warn('"' + path + '" is not a directory.');
                return [];
            }
            for (name in dirObject.items) {
                if (dirObject.items.hasOwnProperty(name)) {
                    if (this.isFile(path + '/' + name)) {
                        fileList.push(name);
                    } else {
                        dirList.push(name);
                    }
                }
            }
            dirList.sort();
            fileList.sort();
            return dirList.concat(fileList);
        };
        /**
         * Returns the list of files of all nested levels located in the specified directory 
         * 
         * @param {String} path A path to the read directory 
         * @param {Boolean} expandArchives An optional parameter. If set to "true" the archives will be expanded.
         *     By default is "false" if the path is a directory or "true" if the path is an archive directory.
         * @returns {Array} An array of child files (all nested levels)
         */
        this.enumerateRecursive = function(path, expandArchives) {
            var fileList = [], pathParts, subChild, child = this.enumerate(path), i;
            if (typeof expandArchives === 'undefined') {
                if (this.isArchive(path)) {
                    expandArchives = true;
                } else {
                    expandArchives = false;
                }
            }
            if (this.isArchive(path) && !expandArchives) {
                return [];
            }
            for (i = 0; i < child.length; i++) {
                if (this.isFile(path + '/' + child[i]) || this.isArchive(path + '/' + child[i]) && !expandArchives) {
                    pathParts = getPathParts(path + '/' + child[i]);
                    fileList.push('/' + pathParts.join('/'));
                } else {
                    subChild = this.enumerateRecursive(path + '/' + child[i], expandArchives);
                    fileList = fileList.concat(subChild);
                }
            }
            return fileList;
        };

        /**
         * Checks if the object in the specified path is a directory 
         * 
         * @param {String} path A path to the object being checked
         * @returns {Boolean} Returns true if the specified path exists and it is directory,otherwise returns "false"
         */
        this.isDirectory = function(path) {
            var file = getObjectByPath(path);
            if (file && file.items) {
                return true;
            } else {
                return false;
            }
        };
        /**
         * Checks if the object in the specified path is a file
         * 
         * @param {String} path A path to the object being checked
         * @returns {Boolean} Returns "true" if the specified path exists and it is a file, otherwise returns "false" 
         */
        this.isFile = function(path) {
            var file = getObjectByPath(path);
            if (file && typeof file.content !== 'undefined') {
                return true;
            } else {
                return false;
            }
        };
        /**
         * Checks if the object in the specified path is a binary file 
         * 
         * @param {String} path A path to the file being checked
         * @returns {Boolean} Returns "true" if the specified path exists and it is a binary file, otherwise returns "false" 
         */
        this.isBinary = function(path) {
            var file = getObjectByPath(path);
            if (file && file.type === 'data') {
                return true;
            } else {
                return false;
            }
        };
        /**
         * Checks if the object in the specified path is an archive
         * 
         * @param {String} path A path to the directory being checked
         * @returns {Boolean} Returns "true" if a specified path exists and it is an archive directory, otherwise returns "false" 
         */
        this.isArchive = function(path) {
            var file = getObjectByPath(path);
            if (file && (file.type === 'zip' || file.type === 'cab')) {
                return true;
            } else {
                return false;
            }
        };
        /**
         * Copies a file or folder recursively to another FileManager object
         * 
         * @param string src A path to the source file or directory
         * @param FileManager destFileManager A destination FileManager object, where the file/directory copy will be stored
         * @param string dest A path to a new file/directory location
         */
        this.copyTo = function(src, destFileManager, dest) {
            var destStorage = null, destParts, destParentDir, srcObj, i, file;
            if (!destFileManager instanceof window.FileManager) {
                console.warn('Cannot find the destination storage.');
            }
            for (i = 0; i < _fileManagerStorages.length; i++) {
                if (_fileManagerStorages[i][0] === destFileManager) {
                    destStorage = _fileManagerStorages[i][1];
                    break;
                }
            }
            if (destStorage === null) {
                console.warn('Cannot find the destination storage.');
            }

            if (this.exists(src)) {
                if (this.isFile(src)) {
                    destParts = getPathParts(dest);
                    if (!destParts.length) {
                        console.warn('Unable to replace the the root directory with "' + src + '" file.');
                        return;
                    }
                    destParentDir = destParts.slice(0, -1).join('/');
                    destFileManager.createDirectory(destParentDir);
                    getObjectByPath(destStorage, destParentDir).items[destParts[destParts.length - 1]] = clone(getObjectByPath(src));
                } else if (this.isDirectory(src)) {
                    srcObj = getObjectByPath(src);
                    if (srcObj.type === 'dir') {
                        destFileManager.createDirectory(dest);
                    } else {
                        destFileManager.createArchive(dest, srcObj.type);
                    }
                    for (file in srcObj.items) {
                        if (srcObj.items.hasOwnProperty(file)) {
                            this.copyTo(src + '/' + file, destFileManager, dest + '/' + file);
                        }
                    }
                }
            } else {
                console.warn('The source "' + src + '" file does not exist.');
                return;
            }
        };
        /**
         * Serializes the file storage into a string to pass to Artisteer
         *
         * @return string
         */
        this.toJSON = function() {
            return JSON.stringify(_fileStorage);
        };
    };
})();


// jQuery outerHTML plugin
jQuery.fn.extend({
    outerHTML: function(value) {
        'use strict';
        if (jQuery(this).length === 0) {
            return typeof value === 'string' ? jQuery(this) : '';
        }
        if (typeof this.outerHTML === 'string') {
            if (typeof value === 'string') {
                jQuery(this).each(function() {
                    this.outerHTML = value;
                });
                return jQuery(this);
            } else {
                return jQuery(this)[0].outerHTML;
            }
        } else {
            if (typeof value === 'string') {
                jQuery(this).replaceWith(value);
                return jQuery(this);
            } else {
                return jQuery(this).eq(0).clone().wrap('<div>').parent().html();
            }
        }
    }
});

var code;

(function () {
    'use strict';

    var values = {};

    code = function (text) {
        var key = '{CODE-' + Math.random() + '}';
        values[key] = text;
        return key;
    };

    code.values = function () {
        return values;
    };
}());

function code(text) {
}